"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Represents a set of modifier tags that were extracted from a doc comment.
 *
 * @remarks
 * TSDoc modifier tags are block tags that do not have any associated rich text content.
 * Instead, their presence or absence acts as an on/off switch, indicating some aspect
 * of the underlying API item.  For example, the `@internal` modifier indicates that a
 * signature is internal (i.e. not part of the public API contract).
 */
var ModifierTagSet = /** @class */ (function () {
    function ModifierTagSet() {
        this._nodes = [];
        // NOTE: To implement case insensitivity, the keys in this set are always upper-case.
        // This convention makes the normalization more obvious (and as a general practice handles
        // the Turkish "i" character correctly).
        this._names = new Set();
    }
    Object.defineProperty(ModifierTagSet.prototype, "nodes", {
        /**
         * The original block tag nodes that defined the modifiers in this set.
         */
        get: function () {
            return this._nodes;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns true if a DocBlockTag with the specified tag name was added to this set.
     * Note that synonyms are not considered.  The comparison is case-insensitive.
     * @param tagName - The name of the tag, including the `@` prefix  For example, `@internal`
     */
    ModifierTagSet.prototype.hasModifierTag = function (modifierTag) {
        return this._names.has(modifierTag.toUpperCase());
    };
    /**
     * Returns true if a DocBlockTag with the specified tag name was added to this set.
     * Note that synonyms are not considered.  The comparison is case-insensitive.
     * @param tagName - The name of the tag, including the `@` prefix  For example, `@internal`
     */
    ModifierTagSet.prototype.hasModifierTagWithUpperCase = function (alreadyUpperCaseModifierTag) {
        return this._names.has(alreadyUpperCaseModifierTag);
    };
    /**
     * Adds a new modifier tag to the set.  If a tag already exists with the same name,
     * then no change is made, and the return value is false.
     */
    ModifierTagSet.prototype.addModifierTag = function (blockTag) {
        if (this._names.has(blockTag.tagNameWithUpperCase)) {
            return false;
        }
        this._names.add(blockTag.tagNameWithUpperCase);
        this._nodes.push(blockTag);
        return true;
    };
    return ModifierTagSet;
}());
exports.ModifierTagSet = ModifierTagSet;
//# sourceMappingURL=ModifierTagSet.js.map