import { DocNode, DocNodeKind, IDocNodeParameters } from './DocNode';
import { ParserContext } from '../parser/ParserContext';
import { DocSection } from './DocSection';
import { StandardModifierTagSet } from '../details/StandardModifierTagSet';
import { DocBlock } from './DocBlock';
import { DocParamBlock } from './DocParamBlock';
/**
 * Constructor parameters for {@link DocComment}.
 */
export interface IDocCommentParameters extends IDocNodeParameters {
    parserContext?: ParserContext;
}
/**
 * Represents an entire documentation comment conforming to the TSDoc structure.
 * This is the root of the DocNode tree.
 */
export declare class DocComment extends DocNode {
    /** {@inheritdoc} */
    readonly kind: DocNodeKind;
    /**
     * The main documentation for an API item is separated into a brief "summary" section,
     * optionally followed by more detailed "remarks" section.
     *
     * @remarks
     * The summary section should be brief. On a documentation web site, it will be shown
     * on a page that lists summaries for many different API items.  On a detail page for
     * a single item, the summary will be shown followed by the remarks section (if any).
     */
    summarySection: DocSection;
    /**
     * The main documentation for an API item is separated into a brief "summary" section
     * followed by more detailed "remarks" section.
     *
     * @remarks
     * Unlike the summary, the remarks block may contain lengthy documentation content.
     * The remarks should not restate information from the summary, since the summary section
     * will always be displayed wherever the remarks section appears.  Other sections
     * (e.g. an `@example` block) will be shown after the remarks section.
     */
    remarksBlock: DocBlock | undefined;
    /**
     * The collection of parsed `@param` blocks for this doc comment.
     */
    paramBlocks: DocParamBlock[];
    /**
     * The `@returns` block for this doc comment, or undefined if there is not one.
     */
    returnsBlock: DocBlock | undefined;
    /**
     * The modifier tags for this DocComment.
     */
    readonly modifierTagSet: StandardModifierTagSet;
    private _customBlocks;
    /**
     * Don't call this directly.  Instead use {@link TSDocParser}
     * @internal
     */
    constructor(parameters: IDocCommentParameters);
    /**
     * The collection of all DocBlock nodes belonging to this doc comment.
     */
    readonly customBlocks: ReadonlyArray<DocBlock>;
    /**
     * Append an item to the customBlocks collection.
     */
    appendCustomBlock(block: DocBlock): void;
    /**
     * {@inheritdoc}
     * @override
     */
    getChildNodes(): ReadonlyArray<DocNode>;
}
