"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Token_1 = require("./Token");
var Tokenizer_1 = require("./Tokenizer");
var nodes_1 = require("../nodes");
var TokenSequence_1 = require("./TokenSequence");
var Excerpt_1 = require("./Excerpt");
var TokenReader_1 = require("./TokenReader");
var StringChecks_1 = require("./StringChecks");
var TSDocParserConfiguration_1 = require("./TSDocParserConfiguration");
var StandardTags_1 = require("../details/StandardTags");
function isFailure(resultOrFailure) {
    return resultOrFailure !== undefined && resultOrFailure.hasOwnProperty('failureMessage');
}
/**
 * The main parser for TSDoc comments.
 */
var NodeParser = /** @class */ (function () {
    function NodeParser(parserContext) {
        this._parserContext = parserContext;
        this._tokenReader = new TokenReader_1.TokenReader(parserContext);
        this._verbatimNodes = parserContext.verbatimNodes;
        this._currentSection = parserContext.docComment.summarySection;
    }
    NodeParser.prototype.parse = function () {
        var done = false;
        while (!done) {
            // Extract the next token
            switch (this._tokenReader.peekTokenKind()) {
                case Token_1.TokenKind.EndOfInput:
                    done = true;
                    break;
                case Token_1.TokenKind.Newline:
                    this._pushAccumulatedPlainText();
                    this._tokenReader.readToken();
                    this._pushDocNode(new nodes_1.DocSoftBreak({
                        excerpt: new Excerpt_1.Excerpt({ content: this._tokenReader.extractAccumulatedSequence() })
                    }));
                    break;
                case Token_1.TokenKind.Backslash:
                    this._pushAccumulatedPlainText();
                    this._pushDocNode(this._parseBackslashEscape());
                    break;
                case Token_1.TokenKind.AtSign:
                    this._pushAccumulatedPlainText();
                    this._parseAndPushBlock();
                    break;
                case Token_1.TokenKind.LeftCurlyBracket:
                    this._pushAccumulatedPlainText();
                    this._pushDocNode(this._parseInlineTag());
                    break;
                case Token_1.TokenKind.RightCurlyBracket:
                    this._pushAccumulatedPlainText();
                    this._pushDocNode(this._createError('The "}" character should be escaped using a backslash to avoid confusion with a TSDoc inline tag'));
                    break;
                case Token_1.TokenKind.LessThan:
                    this._pushAccumulatedPlainText();
                    // Look ahead two tokens to see if this is "<a>" or "</a>".
                    if (this._tokenReader.peekTokenAfterKind() === Token_1.TokenKind.Slash) {
                        this._pushDocNode(this._parseHtmlEndTag());
                    }
                    else {
                        this._pushDocNode(this._parseHtmlStartTag());
                    }
                    break;
                case Token_1.TokenKind.GreaterThan:
                    this._pushAccumulatedPlainText();
                    this._pushDocNode(this._createError('The ">" character should be escaped using a backslash to avoid confusion with an HTML tag'));
                    break;
                case Token_1.TokenKind.Backtick:
                    this._pushAccumulatedPlainText();
                    this._pushDocNode(this._parseCodeSpan());
                    break;
                default:
                    // If nobody recognized this token, then accumulate plain text
                    this._tokenReader.readToken();
                    break;
            }
        }
        this._pushAccumulatedPlainText();
    };
    NodeParser.prototype._pushAccumulatedPlainText = function () {
        if (!this._tokenReader.isAccumulatedSequenceEmpty()) {
            var plainTextSequence = this._tokenReader.extractAccumulatedSequence();
            this._pushDocNode(new nodes_1.DocPlainText({
                text: plainTextSequence.toString(),
                excerpt: new Excerpt_1.Excerpt({ content: plainTextSequence })
            }));
        }
    };
    NodeParser.prototype._parseAndPushBlock = function () {
        var docComment = this._parserContext.docComment;
        var configuration = this._parserContext.configuration;
        var modifierTagSet = docComment.modifierTagSet;
        var parsedBlockTag = this._parseBlockTag();
        if (parsedBlockTag.kind !== "BlockTag" /* BlockTag */) {
            this._pushDocNode(parsedBlockTag);
            return;
        }
        var docBlockTag = parsedBlockTag;
        // Do we have a definition for this tag?
        var tagDefinition = configuration.tryGetTagDefinitionWithUpperCase(docBlockTag.tagNameWithUpperCase);
        if (tagDefinition) {
            switch (tagDefinition.syntaxKind) {
                case TSDocParserConfiguration_1.TSDocTagSyntaxKind.BlockTag:
                    if (docBlockTag.tagNameWithUpperCase === StandardTags_1.StandardTags.param.tagNameWithUpperCase) {
                        var docParamBlock = this._parseParamBlock(docBlockTag);
                        this._parserContext.docComment.paramBlocks.push(docParamBlock);
                        this._currentSection = docParamBlock;
                        return;
                    }
                    else {
                        var newBlock = new nodes_1.DocBlock({
                            blockTag: docBlockTag
                        });
                        this._addBlockToDocComment(newBlock);
                        this._currentSection = newBlock;
                        // But for the verbatimNodes, add the DocBlockTag directly without folding it
                        // into a DocBlock
                        this._verbatimNodes.push(docBlockTag);
                    }
                    return;
                case TSDocParserConfiguration_1.TSDocTagSyntaxKind.ModifierTag:
                    // The block tag was recognized as a modifier, so add it to the modifier tag set
                    // and do NOT call currentSection.appendNode(parsedNode)
                    modifierTagSet.addModifierTag(docBlockTag);
                    this._verbatimNodes.push(docBlockTag);
                    return;
            }
        }
        this._pushDocNode(docBlockTag);
    };
    NodeParser.prototype._addBlockToDocComment = function (block) {
        var docComment = this._parserContext.docComment;
        switch (block.blockTag.tagNameWithUpperCase) {
            case StandardTags_1.StandardTags.remarks.tagNameWithUpperCase:
                docComment.remarksBlock = block;
                break;
            case StandardTags_1.StandardTags.returns.tagNameWithUpperCase:
                docComment.returnsBlock = block;
                break;
            default:
                docComment.appendCustomBlock(block);
        }
    };
    NodeParser.prototype._parseParamBlock = function (docBlockTag) {
        var startMarker = this._tokenReader.createMarker();
        this._readSpacingAndNewlines();
        var leadingWhitespaceSequence = this._tokenReader.tryExtractAccumulatedSequence();
        var parameterName = '';
        var done = false;
        while (!done) {
            switch (this._tokenReader.peekTokenKind()) {
                case Token_1.TokenKind.AsciiWord:
                case Token_1.TokenKind.Period:
                    parameterName += this._tokenReader.readToken();
                    break;
                default:
                    done = true;
                    break;
            }
        }
        if (parameterName.length === 0) {
            this._tokenReader.backtrackToMarker(startMarker);
            var errorParamBlock = new nodes_1.DocParamBlock({
                blockTag: docBlockTag,
                parameterName: ''
            });
            this._parserContext.log.addMessageForTokenSequence('The @param block should be followed by a parameter name', docBlockTag.excerpt.content, docBlockTag);
            return errorParamBlock;
        }
        var parameterNameExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // TODO: Warn if there is no space before or after the hyphen
        this._readSpacingAndNewlines();
        parameterNameExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.Hyphen) {
            this._tokenReader.backtrackToMarker(startMarker);
            this._parserContext.log.addMessageForTokenSequence('The @param block should be followed by a parameter name and then a hyphen', docBlockTag.excerpt.content, docBlockTag);
            return new nodes_1.DocParamBlock({
                blockTag: docBlockTag,
                parameterName: ''
            });
        }
        this._tokenReader.readToken();
        var hyphenExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // TODO: Only read one space
        this._readSpacingAndNewlines();
        hyphenExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        if (leadingWhitespaceSequence) {
            // The leading whitespace that we parsed to the docBlockTag
            docBlockTag.excerpt = new Excerpt_1.Excerpt({
                content: docBlockTag.excerpt.content,
                spacingAfterContent: leadingWhitespaceSequence
            });
        }
        return new nodes_1.DocParamBlock({
            blockTag: docBlockTag,
            parameterNameExcerpt: new Excerpt_1.Excerpt(parameterNameExcerptParameters),
            parameterName: parameterName,
            hyphenExcerpt: new Excerpt_1.Excerpt(hyphenExcerptParameters)
        });
    };
    NodeParser.prototype._pushDocNode = function (docNode) {
        this._currentSection.appendNode(docNode);
        this._verbatimNodes.push(docNode);
    };
    NodeParser.prototype._parseBackslashEscape = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        this._tokenReader.readToken();
        if (this._tokenReader.peekTokenKind() === Token_1.TokenKind.EndOfInput) {
            return this._backtrackAndCreateError(marker, 'A backslash must precede another character that is being escaped');
        }
        var escapedToken = this._tokenReader.readToken();
        // In CommonMark, a backslash is only allowed before a punctuation
        // character.  In all other contexts, the backslash is interpreted as a
        // literal character.
        if (!Tokenizer_1.Tokenizer.isPunctuation(escapedToken.kind)) {
            return this._backtrackAndCreateError(marker, 'A backslash can only be used to escape a punctuation character');
        }
        var tokenSequence = this._tokenReader.extractAccumulatedSequence();
        return new nodes_1.DocEscapedText({
            excerpt: new Excerpt_1.Excerpt({ content: tokenSequence }),
            escapeStyle: nodes_1.EscapeStyle.CommonMarkBackslash,
            text: escapedToken.toString()
        });
    };
    NodeParser.prototype._parseBlockTag = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.AtSign) {
            return this._backtrackAndCreateError(marker, 'Expecting a TSDoc tag starting with "@"');
        }
        // "@one" is a valid TSDoc tag at the start of a line, but "@one@two" is
        // a syntax error.  For two tags it should be "@one @two", or for literal text it
        // should be "\@one\@two".
        switch (this._tokenReader.peekPreviousTokenKind()) {
            case Token_1.TokenKind.None:
            case Token_1.TokenKind.Spacing:
            case Token_1.TokenKind.Newline:
                break;
            default:
                return this._backtrackAndCreateError(marker, 'A TSDoc tag must be preceded by whitespace');
        }
        // Include the "@" as part of the tagName
        var tagName = this._tokenReader.readToken().toString();
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.AsciiWord) {
            return this._backtrackAndCreateError(marker, 'Expecting a TSDoc tag name after the "@" character (or use a backslash to escape this character)');
        }
        var tagNameMarker = this._tokenReader.createMarker();
        while (this._tokenReader.peekTokenKind() === Token_1.TokenKind.AsciiWord) {
            tagName += this._tokenReader.readToken().toString();
        }
        if (tagName.length === 0) {
            return this._backtrackAndCreateError(marker, 'Expecting an inline TSDoc tag name immediately after "{@"');
        }
        if (StringChecks_1.StringChecks.explainIfNotTSDocTagName(tagName)) {
            var failure = this._createFailureForTokensSince('A TSDoc tag name must start with a letter and contain only letters and numbers', tagNameMarker);
            return this._backtrackAndCreateErrorForFailure(marker, '', failure);
        }
        switch (this._tokenReader.peekTokenKind()) {
            case Token_1.TokenKind.None:
            case Token_1.TokenKind.Spacing:
            case Token_1.TokenKind.Newline:
            case Token_1.TokenKind.EndOfInput:
                break;
            default:
                return this._backtrackAndCreateError(marker, 'A TSDoc tag must be followed by whitespace');
        }
        return new nodes_1.DocBlockTag({
            excerpt: new Excerpt_1.Excerpt({ content: this._tokenReader.extractAccumulatedSequence() }),
            tagName: tagName
        });
    };
    NodeParser.prototype._parseInlineTag = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.LeftCurlyBracket) {
            return this._backtrackAndCreateError(marker, 'Expecting a TSDoc tag starting with "{"');
        }
        this._tokenReader.readToken();
        var openingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // For inline tags, if we handle errors by backtracking to the "{"  token, then the main loop
        // will then interpret the "@" as a block tag, which is almost certainly incorrect.  So the
        // DocErrorText needs to include both the "{" and "@" tokens.
        // We will use _backtrackAndCreateErrorRangeForFailure() for that.
        var atSignMarker = this._tokenReader.createMarker();
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.AtSign) {
            return this._backtrackAndCreateError(marker, 'Expecting a TSDoc tag starting with "{@"');
        }
        // Include the "@" as part of the tagName
        var tagName = this._tokenReader.readToken().toString();
        while (this._tokenReader.peekTokenKind() === Token_1.TokenKind.AsciiWord) {
            tagName += this._tokenReader.readToken().toString();
        }
        if (tagName === '@') {
            // This is an unusual case
            var failure = this._createFailureForTokensSince('Expecting a TSDoc inline tag name after the "{@" characters', atSignMarker);
            return this._backtrackAndCreateErrorRangeForFailure(marker, atSignMarker, '', failure);
        }
        if (StringChecks_1.StringChecks.explainIfNotTSDocTagName(tagName)) {
            var failure = this._createFailureForTokensSince('A TSDoc tag name must start with a letter and contain only letters and numbers', atSignMarker);
            return this._backtrackAndCreateErrorRangeForFailure(marker, atSignMarker, '', failure);
        }
        var tagNameExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // We include the space in tagContent in case the implementor wants to assign some
        // special meaning to spaces for their tag.
        var tagContent = this._readSpacingAndNewlines();
        if (tagContent.length === 0) {
            // If there were no spaces at all, that's an error unless it's the degenerate "{@tag}" case
            if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.RightCurlyBracket) {
                var failure = this._createFailureForToken('Expecting a space after the TSDoc inline tag name');
                return this._backtrackAndCreateErrorRangeForFailure(marker, atSignMarker, '', failure);
            }
        }
        var done = false;
        while (!done) {
            switch (this._tokenReader.peekTokenKind()) {
                case Token_1.TokenKind.EndOfInput:
                    return this._backtrackAndCreateErrorRange(marker, atSignMarker, 'The TSDoc inline tag name is missing its closing "}"');
                case Token_1.TokenKind.Backslash:
                    // http://usejsdoc.org/about-block-inline-tags.html
                    // "If your tag's text includes a closing curly brace (}), you must escape it with
                    // a leading backslash (\)."
                    this._tokenReader.readToken(); // discard the backslash
                    // In CommonMark, a backslash is only allowed before a punctuation
                    // character.  In all other contexts, the backslash is interpreted as a
                    // literal character.
                    if (!Tokenizer_1.Tokenizer.isPunctuation(this._tokenReader.peekTokenKind())) {
                        var failure = this._createFailureForToken('A backslash can only be used to escape a punctuation character');
                        return this._backtrackAndCreateErrorRangeForFailure(marker, atSignMarker, 'Error reading inline TSDoc tag: ', failure);
                    }
                    tagContent += this._tokenReader.readToken().toString();
                    break;
                case Token_1.TokenKind.LeftCurlyBracket:
                    {
                        var failure = this._createFailureForToken('The "{" character must be escaped with a backslash when used inside a TSDoc inline tag');
                        return this._backtrackAndCreateErrorRangeForFailure(marker, atSignMarker, '', failure);
                    }
                case Token_1.TokenKind.RightCurlyBracket:
                    done = true;
                    break;
                default:
                    tagContent += this._tokenReader.readToken().toString();
                    break;
            }
        }
        var tagContentExcerpt;
        if (!this._tokenReader.isAccumulatedSequenceEmpty()) {
            tagContentExcerpt = new Excerpt_1.Excerpt({
                content: this._tokenReader.extractAccumulatedSequence()
            });
        }
        // Read the right curly bracket
        this._tokenReader.readToken();
        var closingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        return new nodes_1.DocInlineTag({
            openingDelimiterExcerpt: new Excerpt_1.Excerpt(openingDelimiterExcerptParameters),
            tagNameExcerpt: new Excerpt_1.Excerpt(tagNameExcerptParameters),
            tagName: tagName,
            tagContentExcerpt: tagContentExcerpt,
            tagContent: tagContent,
            closingDelimiterExcerpt: new Excerpt_1.Excerpt(closingDelimiterExcerptParameters)
        });
    };
    NodeParser.prototype._parseHtmlStartTag = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        // Read the "<" delimiter
        var lessThanToken = this._tokenReader.readToken();
        if (lessThanToken.kind !== Token_1.TokenKind.LessThan) {
            return this._backtrackAndCreateError(marker, 'Expecting an HTML tag starting with "<"');
        }
        // NOTE: CommonMark does not permit whitespace after the "<"
        var openingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // Read the element name
        var elementName = this._parseHtmlName();
        if (isFailure(elementName)) {
            return this._backtrackAndCreateErrorForFailure(marker, 'Invalid HTML element: ', elementName);
        }
        var elementNameExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        var spacingAfterElementName = this._readSpacingAndNewlines();
        elementNameExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        var htmlAttributes = [];
        // Read the attributes until we see a ">" or "/>"
        while (this._tokenReader.peekTokenKind() === Token_1.TokenKind.AsciiWord) {
            // Read the attribute
            var attributeNode = this._parseHtmlAttribute();
            if (isFailure(attributeNode)) {
                return this._backtrackAndCreateErrorForFailure(marker, 'The HTML element has an invalid attribute: ', attributeNode);
            }
            htmlAttributes.push(attributeNode);
        }
        // Read the closing "/>" or ">" as the Excerpt.suffix
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var endDelimiterMarker = this._tokenReader.createMarker();
        var selfClosingTag = false;
        if (this._tokenReader.peekTokenKind() === Token_1.TokenKind.Slash) {
            this._tokenReader.readToken();
            selfClosingTag = true;
        }
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.GreaterThan) {
            var failure = this._createFailureForTokensSince('Expecting an attribute or ">" or "/>"', endDelimiterMarker);
            return this._backtrackAndCreateErrorForFailure(marker, 'The HTML tag has invalid syntax: ', failure);
        }
        this._tokenReader.readToken();
        var closingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // NOTE: We don't read excerptParameters.separator here, since if there is any it
        // will be represented as DocPlainText.
        return new nodes_1.DocHtmlStartTag({
            openingDelimiterExcerpt: new Excerpt_1.Excerpt(openingDelimiterExcerptParameters),
            elementNameExcerpt: new Excerpt_1.Excerpt(elementNameExcerptParameters),
            elementName: elementName,
            spacingAfterElementName: spacingAfterElementName,
            htmlAttributes: htmlAttributes,
            selfClosingTag: selfClosingTag,
            closingDelimiterExcerpt: new Excerpt_1.Excerpt(closingDelimiterExcerptParameters)
        });
    };
    NodeParser.prototype._parseHtmlAttribute = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        // Read the attribute name
        var attributeName = this._parseHtmlName();
        if (isFailure(attributeName)) {
            return attributeName;
        }
        var attributeNameExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        var spacingAfterAttributeName = this._readSpacingAndNewlines();
        attributeNameExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        // Read the equals
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.Equals) {
            return this._createFailureForToken('Expecting "=" after HTML attribute name');
        }
        this._tokenReader.readToken();
        var equalsExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        var spacingAfterEquals = this._readSpacingAndNewlines();
        equalsExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        // Read the attribute value
        var attributeValue = this._parseHtmlString();
        if (isFailure(attributeValue)) {
            return attributeValue;
        }
        var attributeValueExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        var spacingAfterAttributeValue = this._readSpacingAndNewlines();
        attributeValueExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        return new nodes_1.DocHtmlAttribute({
            attributeNameExcerpt: new Excerpt_1.Excerpt(attributeNameExcerptParameters),
            attributeName: attributeName,
            spacingAfterAttributeName: spacingAfterAttributeName,
            equalsExcerpt: new Excerpt_1.Excerpt(equalsExcerptParameters),
            spacingAfterEquals: spacingAfterEquals,
            attributeValueExcerpt: new Excerpt_1.Excerpt(attributeValueExcerptParameters),
            attributeValue: attributeValue,
            spacingAfterAttributeValue: spacingAfterAttributeValue
        });
    };
    NodeParser.prototype._parseHtmlString = function () {
        var marker = this._tokenReader.createMarker();
        var quoteTokenKind = this._tokenReader.peekTokenKind();
        if (quoteTokenKind !== Token_1.TokenKind.DoubleQuote && quoteTokenKind !== Token_1.TokenKind.SingleQuote) {
            return this._createFailureForToken('Expecting an HTML string starting with a single-quote or double-quote character');
        }
        this._tokenReader.readToken();
        var textWithoutQuotes = '';
        while (true) {
            var peekedTokenKind = this._tokenReader.peekTokenKind();
            // Did we find the matching token?
            if (peekedTokenKind === quoteTokenKind) {
                this._tokenReader.readToken(); // extract the quote
                break;
            }
            if (peekedTokenKind === Token_1.TokenKind.EndOfInput || peekedTokenKind === Token_1.TokenKind.Newline) {
                return this._createFailureForToken('The HTML string is missing its closing quote', marker);
            }
            textWithoutQuotes += this._tokenReader.readToken().toString();
        }
        // The next attribute cannot start immedaitely after this one
        if (this._tokenReader.peekTokenKind() === Token_1.TokenKind.AsciiWord) {
            return this._createFailureForToken('The next character after a closing quote must be spacing or punctuation');
        }
        return textWithoutQuotes;
    };
    NodeParser.prototype._parseHtmlEndTag = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        // Read the "</" delimiter
        var lessThanToken = this._tokenReader.peekToken();
        if (lessThanToken.kind !== Token_1.TokenKind.LessThan) {
            return this._backtrackAndCreateError(marker, 'Expecting an HTML tag starting with "</"');
        }
        this._tokenReader.readToken();
        var slashToken = this._tokenReader.peekToken();
        if (slashToken.kind !== Token_1.TokenKind.Slash) {
            return this._backtrackAndCreateError(marker, 'Expecting an HTML tag starting with "</"');
        }
        this._tokenReader.readToken();
        // NOTE: Spaces are not permitted here
        // https://www.w3.org/TR/html5/syntax.html#end-tags
        var openingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        // Read the tag name
        var elementName = this._parseHtmlName();
        if (isFailure(elementName)) {
            return this._backtrackAndCreateErrorForFailure(marker, 'Expecting an HTML element name: ', elementName);
        }
        var elementNameExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        this._readSpacingAndNewlines();
        elementNameExcerptParameters.spacingAfterContent = this._tokenReader.tryExtractAccumulatedSequence();
        // Read the closing ">"
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.GreaterThan) {
            var failure = this._createFailureForToken('Expecting a closing ">" for the HTML tag');
            return this._backtrackAndCreateErrorForFailure(marker, '', failure);
        }
        this._tokenReader.readToken();
        var closingDelimiterExcerptParameters = {
            content: this._tokenReader.extractAccumulatedSequence()
        };
        return new nodes_1.DocHtmlEndTag({
            openingDelimiterExcerpt: new Excerpt_1.Excerpt(openingDelimiterExcerptParameters),
            elementNameExcerpt: new Excerpt_1.Excerpt(elementNameExcerptParameters),
            elementName: elementName,
            closingDelimiterExcerpt: new Excerpt_1.Excerpt(closingDelimiterExcerptParameters)
        });
    };
    /**
     * Parses an HTML name such as an element name or attribute name.
     */
    NodeParser.prototype._parseHtmlName = function () {
        var htmlName = '';
        var marker = this._tokenReader.createMarker();
        if (this._tokenReader.peekTokenKind() === Token_1.TokenKind.Spacing) {
            return this._createFailureForTokensSince('A space is not allowed here', marker);
        }
        var done = false;
        while (!done) {
            switch (this._tokenReader.peekTokenKind()) {
                case Token_1.TokenKind.AsciiWord:
                case Token_1.TokenKind.Hyphen:
                    htmlName += this._tokenReader.readToken().toString();
                    break;
                default:
                    done = true;
                    break;
            }
        }
        if (htmlName.length === 0) {
            return this._createFailureForToken('Expecting an HTML name');
        }
        if (!NodeParser.htmlNameRegExp.test(htmlName)) {
            return this._createFailureForTokensSince('An HTML name must be a sequence of letters separated by hyphens', marker);
        }
        return htmlName;
    };
    NodeParser.prototype._parseCodeSpan = function () {
        this._tokenReader.assertAccumulatedSequenceIsEmpty();
        var marker = this._tokenReader.createMarker();
        // Parse the opening backtick
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.Backtick) {
            return this._createError('Expecting a code span starting with a backtick character "`"');
        }
        switch (this._tokenReader.peekPreviousTokenKind()) {
            case Token_1.TokenKind.Spacing:
            case Token_1.TokenKind.Newline:
            case Token_1.TokenKind.None:
                break;
            default:
                return this._createError('The opening backtick for a code span must be preceded by whitespace');
        }
        this._tokenReader.readToken(); // read the backtick
        var text = '';
        var closingBacktickMarker;
        // Parse the content backtick
        while (true) {
            var peekedTokenKind = this._tokenReader.peekTokenKind();
            // Did we find the matching token?
            if (peekedTokenKind === Token_1.TokenKind.Backtick) {
                closingBacktickMarker = this._tokenReader.createMarker();
                this._tokenReader.readToken();
                break;
            }
            if (peekedTokenKind === Token_1.TokenKind.EndOfInput || peekedTokenKind === Token_1.TokenKind.Newline) {
                return this._backtrackAndCreateError(marker, 'The code span is missing its closing backtick');
            }
            text += this._tokenReader.readToken().toString();
        }
        // Make sure there's whitespace after
        switch (this._tokenReader.peekTokenKind()) {
            case Token_1.TokenKind.Spacing:
            case Token_1.TokenKind.EndOfInput:
            case Token_1.TokenKind.Newline:
                break;
            default:
                var failure = this._createFailureForToken('The closing backtick for a code span must be followed by whitespace', closingBacktickMarker);
                return this._backtrackAndCreateErrorForFailure(marker, 'Error parsing code span: ', failure);
        }
        return new nodes_1.DocCodeSpan({
            excerpt: new Excerpt_1.Excerpt({ content: this._tokenReader.extractAccumulatedSequence() }),
            text: text
        });
    };
    NodeParser.prototype._readSpacingAndNewlines = function () {
        var result = '';
        var done = false;
        do {
            switch (this._tokenReader.peekTokenKind()) {
                case Token_1.TokenKind.Spacing:
                case Token_1.TokenKind.Newline:
                    result += this._tokenReader.readToken().toString();
                    break;
                default:
                    done = true;
                    break;
            }
        } while (!done);
        return result;
    };
    /**
     * Read the next token, and report it as a DocErrorText node.
     */
    NodeParser.prototype._createError = function (errorMessage) {
        this._tokenReader.readToken();
        var tokenSequence = this._tokenReader.extractAccumulatedSequence();
        var docErrorText = new nodes_1.DocErrorText({
            excerpt: new Excerpt_1.Excerpt({ content: tokenSequence }),
            text: tokenSequence.toString(),
            errorMessage: errorMessage,
            errorLocation: tokenSequence
        });
        this._parserContext.log.addMessageForDocErrorText(docErrorText);
        return docErrorText;
    };
    /**
     * Rewind to the specified marker, read the next token, and report it as a DocErrorText node.
     */
    NodeParser.prototype._backtrackAndCreateError = function (marker, errorMessage) {
        this._tokenReader.backtrackToMarker(marker);
        return this._createError(errorMessage);
    };
    /**
     * Rewind to the errorStartMarker, read the tokens up to and including errorInclusiveEndMarker,
     * and report it as a DocErrorText node.
     */
    NodeParser.prototype._backtrackAndCreateErrorRange = function (errorStartMarker, errorInclusiveEndMarker, errorMessage) {
        this._tokenReader.backtrackToMarker(errorStartMarker);
        while (this._tokenReader.createMarker() !== errorInclusiveEndMarker) {
            this._tokenReader.readToken();
        }
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.EndOfInput) {
            this._tokenReader.readToken();
        }
        var tokenSequence = this._tokenReader.extractAccumulatedSequence();
        var docErrorText = new nodes_1.DocErrorText({
            excerpt: new Excerpt_1.Excerpt({ content: tokenSequence }),
            text: tokenSequence.toString(),
            errorMessage: errorMessage,
            errorLocation: tokenSequence
        });
        this._parserContext.log.addMessageForDocErrorText(docErrorText);
        return docErrorText;
    };
    /**
     * Rewind to the specified marker, read the next token, and report it as a DocErrorText node
     * whose location is based on an IFailure.
     */
    NodeParser.prototype._backtrackAndCreateErrorForFailure = function (marker, errorMessagePrefix, failure) {
        this._tokenReader.backtrackToMarker(marker);
        this._tokenReader.readToken();
        var tokenSequence = this._tokenReader.extractAccumulatedSequence();
        var docErrorText = new nodes_1.DocErrorText({
            excerpt: new Excerpt_1.Excerpt({ content: tokenSequence }),
            text: tokenSequence.toString(),
            errorMessage: errorMessagePrefix + failure.failureMessage,
            errorLocation: failure.failureLocation
        });
        this._parserContext.log.addMessageForDocErrorText(docErrorText);
        return docErrorText;
    };
    /**
     * Rewind to the errorStartMarker, read the tokens up to and including errorInclusiveEndMarker,
     * and report it as a DocErrorText node whose location is based on an IFailure.
     */
    NodeParser.prototype._backtrackAndCreateErrorRangeForFailure = function (errorStartMarker, errorInclusiveEndMarker, errorMessagePrefix, failure) {
        this._tokenReader.backtrackToMarker(errorStartMarker);
        while (this._tokenReader.createMarker() !== errorInclusiveEndMarker) {
            this._tokenReader.readToken();
        }
        if (this._tokenReader.peekTokenKind() !== Token_1.TokenKind.EndOfInput) {
            this._tokenReader.readToken();
        }
        var tokenSequence = this._tokenReader.extractAccumulatedSequence();
        var docErrorText = new nodes_1.DocErrorText({
            excerpt: new Excerpt_1.Excerpt({ content: tokenSequence }),
            text: tokenSequence.toString(),
            errorMessage: errorMessagePrefix + failure.failureMessage,
            errorLocation: failure.failureLocation
        });
        this._parserContext.log.addMessageForDocErrorText(docErrorText);
        return docErrorText;
    };
    /**
     * Creates an IFailure whose TokenSequence is a single token.  If a marker is not specified,
     * then it is the current token.
     */
    NodeParser.prototype._createFailureForToken = function (failureMessage, tokenMarker) {
        if (!tokenMarker) {
            tokenMarker = this._tokenReader.createMarker();
        }
        var tokenSequence = new TokenSequence_1.TokenSequence({
            parserContext: this._parserContext,
            startIndex: tokenMarker,
            endIndex: tokenMarker + 1
        });
        return {
            failureMessage: failureMessage,
            failureLocation: tokenSequence
        };
    };
    /**
     * Creates an IFailure whose TokenSequence starts from the specified marker and
     * encompasses all tokens read since then.  If none were read, then the next token used.
     */
    NodeParser.prototype._createFailureForTokensSince = function (failureMessage, startMarker) {
        var endMarker = this._tokenReader.createMarker();
        if (endMarker < startMarker) {
            // This would be a parser bug
            throw new Error('Invalid startMarker');
        }
        if (endMarker === startMarker) {
            ++endMarker;
        }
        var tokenSequence = new TokenSequence_1.TokenSequence({
            parserContext: this._parserContext,
            startIndex: startMarker,
            endIndex: endMarker
        });
        return {
            failureMessage: failureMessage,
            failureLocation: tokenSequence
        };
    };
    // https://www.w3.org/TR/html5/syntax.html#tag-name
    // https://html.spec.whatwg.org/multipage/custom-elements.html#valid-custom-element-name
    NodeParser.htmlNameRegExp = /^[a-z]+(\-[a-z]+)*$/i;
    return NodeParser;
}());
exports.NodeParser = NodeParser;
//# sourceMappingURL=NodeParser.js.map